/* SD card and FAT filesystem example.
   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/

#include <stdio.h>
#include <string.h>
#include <sys/unistd.h>
#include <sys/stat.h>
#include "esp_err.h"
#include "esp_log.h"
#include "esp_vfs_fat.h"
#include "driver/sdmmc_host.h"
#include "driver/sdspi_host.h"
#include "sdmmc_cmd.h"
#include "driver/uart.h"
#include "soc/uart_struct.h"


static const char *TAG = "example";

// This example can use SDMMC and SPI peripherals to communicate with SD card.
// By default, SDMMC peripheral is used.
// To enable SPI mode, uncomment the following line:

#define USE_SPI_MODE

// When testing SD and SPI modes, keep in mind that once the card has been
// initialized in SPI mode, it can not be reinitialized in SD mode without
// toggling power to the card.

#ifdef USE_SPI_MODE
// Pin mapping when using SPI mode.
// With this mapping, SD card can be used both in SPI and 1-line SD mode.
// Note that a pull-up on CS line is required in SD mode.
#define PIN_NUM_CLK  16
#define PIN_NUM_MOSI 17
#define PIN_NUM_MISO 18
#define PIN_NUM_CS   19
#endif //USE_SPI_MODE

#define TXD_PIN 22
#define RXD_PIN 23

static const int RX_BUF_SIZE = 1024;

//UART
void init() {
    const uart_config_t uart_config = {
        .baud_rate = 3000000,
        .data_bits = UART_DATA_8_BITS,
        .parity = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE
    };
    uart_param_config(UART_NUM_1, &uart_config);
    uart_set_pin(UART_NUM_1, TXD_PIN, RXD_PIN, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);
    // We won't use a buffer for sending data.
    uart_driver_install(UART_NUM_1, RX_BUF_SIZE * 2, RX_BUF_SIZE * 2, 0, NULL, 0);
}


int sendData(const char* data, size_t len)
{
    //const int len = strlen(data);
    const int txBytes = 0;
    uart_write_bytes(UART_NUM_1, data, len);
     return txBytes;
}

long GetFileSize(const char* filename)
{
    long size;
    FILE *f;

    f = fopen(filename, "rb");
    if (f == NULL) return -1;
    fseek(f, 0, SEEK_END);
    size = ftell(f);
    fclose(f);

    return size;
}

void app_main(void)
{
    ESP_LOGI(TAG, "Initializing SD card");
    init();

#ifndef USE_SPI_MODE
    ESP_LOGI(TAG, "Using SDMMC peripheral");
    sdmmc_host_t host = SDMMC_HOST_DEFAULT();

    // To use 1-line SD mode, uncomment the following line:
    // host.flags = SDMMC_HOST_FLAG_1BIT;

    // This initializes the slot without card detect (CD) and write protect (WP) signals.
    // Modify slot_config.gpio_cd and slot_config.gpio_wp if your board has these signals.
    sdmmc_slot_config_t slot_config = SDMMC_SLOT_CONFIG_DEFAULT();

    // GPIOs 15, 2, 4, 12, 13 should have external 10k pull-ups.
    // Internal pull-ups are not sufficient. However, enabling internal pull-ups
    // does make a difference some boards, so we do that here.
    gpio_set_pull_mode(15, GPIO_PULLUP_ONLY);   // CMD, needed in 4- and 1- line modes
    gpio_set_pull_mode(2, GPIO_PULLUP_ONLY);    // D0, needed in 4- and 1-line modes
    gpio_set_pull_mode(4, GPIO_PULLUP_ONLY);    // D1, needed in 4-line mode only
    gpio_set_pull_mode(12, GPIO_PULLUP_ONLY);   // D2, needed in 4-line mode only
    gpio_set_pull_mode(13, GPIO_PULLUP_ONLY);   // D3, needed in 4- and 1-line modes

#else
    ESP_LOGI(TAG, "Using SPI peripheral");

    sdmmc_host_t host = SDSPI_HOST_DEFAULT();
    sdspi_slot_config_t slot_config = SDSPI_SLOT_CONFIG_DEFAULT();
    slot_config.gpio_miso = PIN_NUM_MISO;
    slot_config.gpio_mosi = PIN_NUM_MOSI;
    slot_config.gpio_sck  = PIN_NUM_CLK;
    slot_config.gpio_cs   = PIN_NUM_CS;
    // This initializes the slot without card detect (CD) and write protect (WP) signals.
    // Modify slot_config.gpio_cd and slot_config.gpio_wp if your board has these signals.
#endif //USE_SPI_MODE

    // Options for mounting the filesystem.
    // If format_if_mount_failed is set to true, SD card will be partitioned and
    // formatted in case when mounting fails.
    esp_vfs_fat_sdmmc_mount_config_t mount_config = {
        .format_if_mount_failed = false,
        .max_files = 5,
        .allocation_unit_size = 16 * 1024
    };

    // Use settings defined above to initialize SD card and mount FAT filesystem.
    // Note: esp_vfs_fat_sdmmc_mount is an all-in-one convenience function.
    // Please check its source code and implement error recovery when developing
    // production applications.
    sdmmc_card_t* card;
    esp_err_t ret = esp_vfs_fat_sdmmc_mount("/sdcard", &host, &slot_config, &mount_config, &card);

    if (ret != ESP_OK) {
        if (ret == ESP_FAIL) {
            ESP_LOGE(TAG, "Failed to mount filesystem. "
                "If you want the card to be formatted, set format_if_mount_failed = true.");
        } else {
            ESP_LOGE(TAG, "Failed to initialize the card (%s). "
                "Make sure SD card lines have pull-up resistors in place.", esp_err_to_name(ret));
        }
        return;
    }

    // Card has been initialized, print its properties
    sdmmc_card_print_info(stdout, card);


    ESP_LOGI(TAG, "Opening file");
        FILE* f2 = fopen("/sdcard/moby.wav", "rb");
    	//FILE* f2 = fopen("/sdcard/slash8b.wav", "rb");
        if (f2 == NULL) {
            ESP_LOGE(TAG, "Failed to open file for writing");
            return;
        }

        //Capturan los datos
        char data[4];
        char data2[2];
        long int val;
        //ChunkID
		//fgets(data, sizeof(data), f2);
		fread(data,sizeof(data),1,f2);
		ESP_LOGI(TAG, "ChunkID: '%s'",data);
		//ChunkSize
		//fgets(data, sizeof(data), f2);
		fread(data,sizeof(data),1,f2);
		val = ((data[3])<<24)+ ((data[2])<<16)+ ((data[1])<<8) + (data[0]);
		ESP_LOGI(TAG, "Chunk Size: '%li'",val);
		//Format
		fread(data,sizeof(data),1,f2);
		ESP_LOGI(TAG, "Format: '%s'",data);
		//Subchunk1ID
		fread(data,sizeof(data),1,f2);
		ESP_LOGI(TAG, "Subchunk1ID: '%s'",data);
		//Subchunk1Size
		fread(data,sizeof(data),1,f2);
		val = ((data[3])<<24)+ ((data[2])<<16)+ ((data[1])<<8) + (data[0]);
		ESP_LOGI(TAG, "Subchunk1Size Size: '%li'",val);
		//Audioformat
		fread(data2,sizeof(data2),1,f2);
		val = ((data2[1])<<8) + (data2[0]);
		ESP_LOGI(TAG, "Audioformat: '%li'",val);
		//NumChannels
		fread(data2,sizeof(data2),1,f2);
		val = ((data2[1])<<8) + (data2[0]);
		ESP_LOGI(TAG, "NumChannels: '%li'",val);
		//SampleRate
		fread(data,sizeof(data),1,f2);
		val = ((data[3])<<24)+ ((data[2])<<16)+ ((data[1])<<8) + (data[0]);
		ESP_LOGI(TAG, "SampleRate: '%li'",val);
		//ByteRate
		fread(data,sizeof(data),1,f2);
		val = ((data[3])<<24)+ ((data[2])<<16)+ ((data[1])<<8) + (data[0]);
		ESP_LOGI(TAG, "ByteRate: '%li'",val);
		//BlockAlign
		fread(data2,sizeof(data2),1,f2);
		val = ((data2[1])<<8) + (data2[0]);
		ESP_LOGI(TAG, "BlockAlign: '%li'",val);
		//BitsPerSample
		fread(data2,sizeof(data2),1,f2);
		val = ((data2[1])<<8) + (data2[0]);
		ESP_LOGI(TAG, "BitsPerSample: '%li'",val);
		//Subchunk2ID
		fread(data,sizeof(data),1,f2);
		ESP_LOGI(TAG, "Subchunk2ID: '%s'",data);
		//Subchunk2Size
		fread(data,sizeof(data),1,f2);
		val = ((data[3])<<24)+ ((data[2])<<16)+ ((data[1])<<8) + (data[0]);
		ESP_LOGI(TAG, "Subchunk2Size: '%li'",val);

		char ini[] = {10};
		sendData(ini, sizeof(ini));

		while(1){
			uint8_t* datau = (uint8_t*) malloc(1);
			const int rxBytes = uart_read_bytes(UART_NUM_1, datau, 1, 1000 / portTICK_RATE_MS);
			if (rxBytes > 0) {
				for(int i=0;i<100;i++){
				char data1[1000];
				fread(data1,sizeof(data1),1,f2);
				sendData(data1, sizeof(data1));
				}

			}
		}


        fclose(f2);
    //////////////

    // All done, unmount partition and disable SDMMC or SPI peripheral
    esp_vfs_fat_sdmmc_unmount();
    ESP_LOGI(TAG, "Card unmounted");
}
